#ifndef __BLOCK_CACHE_H
#define __BLOCK_CACHE_H

#include <types.h>

#define BLOCK_CACHE_DIRTY       0x1
#define BLOCK_CACHE_ACCESSED    0x2

typedef struct {
    void* data;                 // Puntero a memoria al bloque cacheado
    uint64_t number;            // Numero de bloque
    uint64_t flags;             // Flags del bloque cacheado
    uint64_t ref_count;         // Cantidad de referencias, en 0 se asume libre 
} block_cache_entry;

typedef struct {
    block_cache_entry* vec;     // Vector que representa este struct
    uint64_t count;             // Cantidad de entradas que tiene el vector
    uint64_t size_sectors;      // Tamaño de un bloque en la cache en sectores
    uint64_t size_bytes;        // Tamaño de un bloque en la cache en bytes        
} block_cache;

block_cache* block_cache_create(uint64_t sector_size, uint64_t count);

void block_cache_free_blocks(block_cache *cache);
void block_cache_flush(block_cache *vec);
void block_cache_free(block_cache *vec);
void* block_cache_load(block_cache *vec, uint64_t block_number);
void block_cache_unload(block_cache *vec, void *block_ptr);
void block_cache_set_dirty(block_cache *cache, void *block_ptr);
void* block_cache_load_dirty(block_cache *cache, uint64_t block_number);

#endif
