#ifndef __DEFINES_H__
#define __DEFINES_H__

/* GDT */
/* -------------------------------------------------------------------------- */
#define GDT_INDEX_NULL              0
#define GDT_INDEX_CODE_DPL0_32      1
#define GDT_INDEX_DATA_DPL0         2
#define GDT_INDEX_CODE_DPL0_64      3
#define GDT_COUNT                   4

/* Paging */
/* -------------------------------------------------------------------------- */
/* Present, Read/Write, System */
#define PAGE_ATTRIBUTES             0x3
#define PAGE_SIZE                   0x1000

/* Selectores de segmento [TI: 0 | RPL: 0] */
#define GDT_DESC_CODE_DPL0_32       (GDT_INDEX_CODE_DPL0_32 << 3)
#define GDT_DESC_DATA_DPL0          (GDT_INDEX_DATA_DPL0    << 3)
#define GDT_DESC_CODE_DPL0_64       (GDT_INDEX_CODE_DPL0_64 << 3)

/* App Data */
/* -------------------------------------------------------------------------- */
#define APP_DATA_ENTRY_POINT        0x00
#define APP_DATA_RUNNING_STATUS     0x08
#define APP_DATA_STACK_POINTER      0x10
#define APP_DATA_EXIT_CODE          0x18
#define APP_DATA_EXIT_CORE_ID       0x20

/* Video */
/* -------------------------------------------------------------------------- */
#define VIDEO_WIDTH                 80
#define VIDEO_HEIGHT                25
#define VIDEO_SIZE                  VIDEO_WIDTH * VIDEO_HEIGHT * 2

/* Direcciones de memoria */
/* -------------------------------------------------------------------------- */
/* Debajo del primer MiB */
/* -------------------------------------------------------------------------- */

/* Direccion fisica del buffer de video */
#define VIDEO_ADDRESS               0x000B8000
/* Direccion fisica donde va el codigo de los AP
 * Tiene que estar alineada a pagina obligatoriamente */
#define AP_STARTUP_ADDRESS          0x00080000
/* Direccion fisica donde va la tabla de syscalls */
#define SYSCALL_TABLE               0x00081000

/* Por encima del primer MiB */
/* -------------------------------------------------------------------------- */

/* Paginacion 4GiB */
#define KERNEL_PML4T                0x007FA000
#define KERNEL_PDPT                 0x007FB000
#define KERNEL_PDT                  0x007FC000
#define KERNEL_PTT                  0x00800000
/* Llega hasta 0x00FFFFFF */

/* 16 Stacks de 1MiB expand down */
#define KERNEL_STACKS               0x01000000
#define KERNEL_STACK_SIZE           0x00100000
#define KERNEL_STACK_BSP            0x01100000

/* 
 * Memoria libre para reservar
 * 96MB de heap desde 0x2000000 a 0x8000000
 */
#define KERNEL_HEAP_START           0x02000000
#define KERNEL_HEAP_END             0x08000000

/* 
 * La posicion donde se cargan los binarios a ejecutar 
 * En el caso de bochs tiene configurado 256mb de ram, asi que son 128mb
 */
#define KERNEL_EXEC_LOAD_ADDRESS    0x08000000

/*
 * A partir de los 3GB hay hardware mapeado no deberíamos poner nada acá
 */
#define MEMORY_MAPPED_HARDWARE      0xC0000000

#endif
