#ifndef __ELF_H
#define __ELF_H

#include <types.h>

#define INVALID_ELF         -1
#define ERRREADING_ELF      -2
#define BAD_LOADER_ACCESS   -3

//Todas las estructuras están pensadas para ELF64.

//ELF64 file object header
typedef struct {
    char magic[16];
    uint16_t type;
    uint16_t machine;
    uint32_t version;
    uint64_t entry_point;
    uint64_t ph_offset;
    uint64_t sh_offset;
    uint32_t flags;
    uint16_t header_size;
    uint16_t ph_entry_size;
    uint16_t ph_entry_count;
    uint16_t sh_entry_size;
    uint16_t sh_entry_count;
    uint16_t sh_string_table_index;
} __attribute__((__packed__)) elf_header;

#define EI_CLASS 4
#define EI_DATAENC 5

#define EM_386 3

#define ELFCLASSNONE 0
#define ELFCLASS32 1
#define ELFCLASS64 2

#define ELFDATANONE 0
#define ELFDATA2LSB 1
#define ELFDATA2MSB 2

//ELF64 section header entry
typedef struct {
    uint32_t name;
    uint32_t type;
    uint64_t flags;
    uint64_t address;
    uint64_t offset;
    uint64_t size;
    uint32_t link;
    uint32_t info;
    uint64_t address_align;
    uint64_t entry_size;
} __attribute__((__packed__)) elf_sheader;

#define SHN_UNDEF 0

#define ELF_NONE 0
#define ELF_LOAD 1

#define ELF_ATTR_XB 1
#define ELF_ATTR_WB 2
#define ELF_ATTR_RB 4

// ELF64 program header entry
typedef struct {
    uint32_t type;
    uint32_t flags;
    uint64_t offset;
    uint64_t virtual_address;
    uint64_t physical_address;
    uint64_t file_size;
    uint64_t memory_size;
    uint64_t align;
} __attribute__((__packed__)) elf_pheader;

#endif
