#ifndef __i386_H__
#define __i386_H__

#include <types.h>

#define RFLAGS_IF_BIT 9

LS_INLINE void hlt() {
    while (1)
    __asm__ __volatile__ ("cli\n\t"
                          "hlt\n\t");
}

LS_INLINE void cli() {
    __asm__ __volatile__ ("sti");
}

LS_INLINE void sti() {
    __asm__ __volatile__ ("sti");
}

LS_INLINE uint64_t cli_safe() {
    uint64_t rflags;
    __asm__ __volatile__ ("pushfq\n\t"
                          "cli\n\t"
                          "pop %0\n\t"
                          : "=r" (rflags));
    return rflags;
}

LS_INLINE void sti_safe(uint64_t rflags) {
    // Si originalmente habia interrupciones 
    // las volvemos a habilitar
    if(rflags & (1 << RFLAGS_IF_BIT))
        __asm__ __volatile__ ("sti");
}

LS_INLINE uint64_t getRAX() {
    uint64_t val;
    __asm __volatile("movq %%rax,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRBX() {
    uint64_t val;
    __asm __volatile("movq %%rbx,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRCX() {
    uint64_t val;
    __asm __volatile("movq %%rcx,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRDX() {
    uint64_t val;
    __asm __volatile("movq %%rdx,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRSI() {
    uint64_t val;
    __asm __volatile("movq %%rsi,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRDI() {
    uint64_t val;
    __asm __volatile("movq %%rdi,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRBP() {
    uint64_t val;
    __asm __volatile("movq %%rbp,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRSP() {
    uint64_t val;
    __asm __volatile("movq %%rsp,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR8() {
    uint64_t val;
    __asm __volatile("movq %%r8,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR9() {
    uint64_t val;
    __asm __volatile("movq %%r9,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR10() {
    uint64_t val;
    __asm __volatile("movq %%r10,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR11() {
    uint64_t val;
    __asm __volatile("movq %%r11,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR12() {
    uint64_t val;
    __asm __volatile("movq %%r12,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR13() {
    uint64_t val;
    __asm __volatile("movq %%r13,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR14() {
    uint64_t val;
    __asm __volatile("movq %%r14,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getR15() {
    uint64_t val;
    __asm __volatile("movq %%r15,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getCS() {
    uint16_t val;
    __asm __volatile("movs %%cs,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getDS() {
    uint16_t val;
    __asm __volatile("movs %%ds,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getES() {
    uint16_t val;
    __asm __volatile("movs %%es,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getFS() {
    uint16_t val;
    __asm __volatile("movs %%fs,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getGS() {
    uint16_t val;
    __asm __volatile("movs %%gs,%0" : "=r" (val));
    return val;
}

LS_INLINE uint16_t getSS() {
    uint16_t val;
    __asm __volatile("movs %%ss,%0" : "=r" (val));
    return val;
}

LS_INLINE void setCR0(uint64_t val) {
    __asm __volatile("movq %0,%%cr0" : : "r" (val));
}

LS_INLINE uint64_t getCR0() {
    uint64_t val;
    __asm __volatile("movq %%cr0,%0" : "=r" (val));
    return val;
}

LS_INLINE void setCR2(uint64_t val) {
    __asm __volatile("movq %0,%%cr2" : : "r" (val));
}

LS_INLINE uint64_t getCR2() {
    uint64_t val;
    __asm __volatile("movq %%cr2,%0" : "=r" (val));
    return val;
}

LS_INLINE void setCR3(uint64_t val) {
    __asm __volatile("movq %0,%%cr3" : : "r" (val));
}

LS_INLINE uint64_t getCR3() {
    uint64_t val;
    __asm __volatile("movq %%cr3,%0" : "=r" (val));
    return val;
}

LS_INLINE void setCR4(uint64_t val) {
    __asm __volatile("movq %0,%%cr4" : : "r" (val));
}

LS_INLINE uint64_t getCR4() {
    uint64_t val;
    __asm __volatile("movq %%cr4,%0" : "=r" (val));
    return val;
}

LS_INLINE void setCR8(uint64_t val) {
    __asm __volatile("movq %0,%%cr8" : : "r" (val));
}

LS_INLINE uint64_t getCR8() {
    uint64_t val;
    __asm __volatile("movq %%cr8,%0" : "=r" (val));
    return val;
}

LS_INLINE uint64_t getRFLAGS() {
    uint64_t val;
    __asm __volatile("pushfq\n\t" 
    	             "pop %0" : "=r" (val));
    return val;
}

LS_INLINE void setRFLAGS(uint64_t val) {
    __asm __volatile("push %0\n\t" 
    	             "popfq" : : "r" (val));
}

#endif  /* !__i386_H__ */
