#ifndef __IDT_H__
#define __IDT_H__

#include <types.h>

// Campo de atts de la IDT:
// |    	| Descriptor   |           |       |           | 
// |Present	| Privilege	   | 0 D 1 1 0 | 0 0 0 | 0 0 0 0 0 |
// |   		| Level		   |           |       |           |

// Trap
//attrs:
//P=1 present segment
//DPL=00 kernel level
//D=1 32 bits
//1000 1111 0000 0000 => 0x8F00
// | 1 | 0 0 | 0 1 1 1 1 | 0 0 0 | 0 0 0 0 0 | = 0x8F00

//Kernel
// Interrupt
//attrs:
//P=1 present segment
//DPL=00 user level
//D=1 32 bits
//1000 1110 0000 0000 => 0x8E00
// | 1 | 0 0 | 0 1 1 1 0 | 0 0 0 | 0 0 0 0 0 | = 0x8E00

//User
// Interrupt
//attrs:
//P=1 present segment
//DPL=11 user level
//D=1 32 bits
//1110 1110 0000 0000 => 0xEE00
// | 1 | 1 1 | 0 1 1 1 0 | 0 0 0 | 0 0 0 0 0 | = 0xEE00

#define IDT_ENTRIES_COUNT 256

#define KERNEL_TRAP_GATE_TYPE 0x8F00
#define KERNEL_INT_GATE_TYPE 0x8E00
#define SERVICE_INT_GATE_TYPE 0xEE00

// Handler de ignorar interrupciones espurias
// De acuerdo a la APIC spec, los 4 bits de abajo deben estar en 1.
// Lo demas lo seteamos nosotros. 
#define INT_NUM_SPURIOUS        0xFF

#define INT_NUM_CLOCK           32 
#define INT_NUM_KEYBOARD        33
#define INT_NUM_RTC             40

#define INT_NUM_ATA_PRIMARY     46
#define INT_NUM_ATA_SECONDARY   47

#define INT_NUM_EXIT            48

/* Struct de descriptor de IDT */
typedef struct str_idt_descriptor {
    uint16_t  idt_length;
    uint64_t    idt_addr;
} __attribute__((__packed__)) idt_descriptor;

/* Struct de una entrada de la IDT */
typedef struct str_idt_entry_fld {
    uint16_t offset_0_15;
    uint16_t segsel;
    uint16_t attr;
    uint16_t offset_16_31;
    uint32_t offset_32_63;
    uint32_t zeroPadd;
} __attribute__((__packed__, aligned (8))) idt_entry;

void idt_init();

#endif  /* !__IDT_H__ */
