#ifndef __PIC_H__
#define __PIC_H__
#include <types.h>

#define PIC1_PORT 0x20
#define PIC2_PORT 0xA0

// Escribe al CMOS
void cmos_writeb(uint8_t port, uint8_t data);
// Lee del CMOS
uint8_t cmos_readb(uint8_t port);

LS_INLINE void outb(uint16_t port, uint8_t data) {
    __asm __volatile("outb %0, %w1" : : "a" (data), "dN" (port));
}

LS_INLINE uint8_t inb(uint16_t port) {
    uint8_t res;
    __asm __volatile("inb %w1, %0" : "=a" (res) : "dN" (port));
    return res;
}

LS_INLINE void outw(uint16_t port, uint16_t data) {
    __asm __volatile("outw %0, %w1" : : "a" (data), "dN" (port));
}

LS_INLINE uint16_t inw(uint16_t port) {
    uint16_t res;
    __asm __volatile("inw %w1, %0" : "=a" (res) : "dN" (port));
    return res;
}

LS_INLINE void outl(uint16_t port, uint32_t data) {
    __asm __volatile("outl %0, %w1" : : "a" (data), "dN" (port));
}

LS_INLINE uint16_t inl(uint16_t port) {
    uint32_t res;
    __asm __volatile("inl %w1, %0" : "=a" (res) : "dN" (port));
    return res;
}


LS_INLINE void mask_pic() {
    outb(PIC1_PORT+0, 0x11); /* IRQs activas x flanco, cascada, y ICW4 */
    outb(PIC1_PORT+1, 0x20); /* Addr */
    outb(PIC1_PORT+1, 0x04); /* PIC1 Master, Slave ingresa Int.x IRQ2 */
    outb(PIC1_PORT+1, 0x01); /* Modo 8086 */
    outb(PIC1_PORT+1, 0xFF); /* Enmasca todas! */

    outb(PIC2_PORT+0, 0x11); /* IRQs activas x flanco, cascada, y ICW4 */
    outb(PIC2_PORT+1, 0x28); /* Addr */
    outb(PIC2_PORT+1, 0x02); /* PIC2 Slave, ingresa Int x IRQ2 */
    outb(PIC2_PORT+1, 0x01); /* Modo 8086 */
    outb(PIC2_PORT+1, 0xFF); /* Enmasca todas! */
}

#endif	/* !__PIC_H__ */
