#include <defines.h>
#include <gdt.h>
#include <idt.h>
#include <isr.h>

idt_entry idt[IDT_ENTRIES_COUNT];

idt_descriptor IDT_DESC = {.idt_length = sizeof(idt) - 1,
                           .idt_addr = (uint64_t)&idt};

/* Macro with 64 bits code segment */
#define IDT_ENTRY(number, attrs) idt_init_entry(number, &_isr##number, attrs);

static void idt_init_entry(uint64_t number, void* handler, uint16_t attrs) {
    idt[number].segsel = GDT_DESC_CODE_DPL0_64;
    idt[number].attr = attrs;
    idt[number].offset_0_15 = (uint64_t)handler & 0xFFFF;
    idt[number].offset_16_31 = ((uint64_t)handler >> 16) & 0xFFFF;
    idt[number].offset_32_63 = ((uint64_t)handler >> 32) & 0xFFFFFFFF;
    idt[number].zeroPadd = 0x0;
}

void idt_init() {
    /* Division por cero(instr div, idiv) */
    IDT_ENTRY(0, KERNEL_TRAP_GATE_TYPE);
    /* Reservada para intel */
    IDT_ENTRY(1, KERNEL_TRAP_GATE_TYPE);
    /* NMI interrupt */
    IDT_ENTRY(2, KERNEL_TRAP_GATE_TYPE);
    /* Breakpoint (Origen int 3) */
    IDT_ENTRY(3, KERNEL_TRAP_GATE_TYPE);
    /* Overflow (INTO instruction) */
    IDT_ENTRY(4, KERNEL_TRAP_GATE_TYPE);
    /* Bound range exceeded (Solo con instruccion bound) */
    IDT_ENTRY(5, KERNEL_TRAP_GATE_TYPE);
    /* Invalid opcode */
    IDT_ENTRY(6, KERNEL_TRAP_GATE_TYPE);
    /* Device not available (Math Processor) */
    IDT_ENTRY(7, KERNEL_TRAP_GATE_TYPE);
    /* Double Fault
     * [Pushea Error Code 0] */
    IDT_ENTRY(8, KERNEL_TRAP_GATE_TYPE);
    /* Coprocessor segment overrun (Reservada)
       Despues del intel 386 no se genera esta excepción */
    IDT_ENTRY(9, KERNEL_TRAP_GATE_TYPE);
    /* Invalid TSS
     * [Pushea Error Code] */
    IDT_ENTRY(10, KERNEL_TRAP_GATE_TYPE);
    /* Segment not present [Pushea Error Code] */
    IDT_ENTRY(11, KERNEL_TRAP_GATE_TYPE);
    /* Stack segment fault
     * [Pushea Error Code] */
    IDT_ENTRY(12, KERNEL_TRAP_GATE_TYPE);
    /* General protection fault
     * [Pushea Error Code] */
    IDT_ENTRY(13, KERNEL_TRAP_GATE_TYPE);
    /* Page Fault
     * [Pushea Error Code] */
    IDT_ENTRY(14, KERNEL_TRAP_GATE_TYPE);
    /* Reservada por Intel (No usar) */
    IDT_ENTRY(15, KERNEL_TRAP_GATE_TYPE);
    /* x87 FPU ERROR (Math Fault) */
    IDT_ENTRY(16, KERNEL_TRAP_GATE_TYPE);
    /* Memory Alignment check
     * [Pushea Error Code 0] */
    IDT_ENTRY(17, KERNEL_TRAP_GATE_TYPE);
    /* Machine Check */
    IDT_ENTRY(18, KERNEL_TRAP_GATE_TYPE);
    /* SIMD Exception */
    IDT_ENTRY(19, KERNEL_TRAP_GATE_TYPE);
    /* Virtualization Exception */
    IDT_ENTRY(20, KERNEL_TRAP_GATE_TYPE);

    /* User Defined Interrupts (IRQs) */
    /* Clock */
    IDT_ENTRY(32, KERNEL_INT_GATE_TYPE);
    /* Keyboard */
    IDT_ENTRY(33, KERNEL_INT_GATE_TYPE);
    /* Real Time Clock */
    IDT_ENTRY(40, KERNEL_INT_GATE_TYPE);
    /* Primary ATA Hard Disk */
    IDT_ENTRY(46, KERNEL_INT_GATE_TYPE);
    /* Secondary ATA Hard Disk */
    IDT_ENTRY(47, KERNEL_INT_GATE_TYPE);
    /* Exit */
    IDT_ENTRY(48, KERNEL_INT_GATE_TYPE);

    // Evitar interrupciones espurias usando el spurious intr number.
    // Esto se usa para el APIC en el vector.
    idt_init_entry(INT_NUM_SPURIOUS, _isr_spurious, KERNEL_INT_GATE_TYPE);
}
