#include <defines.h>
#include <i386.h>
#include <keyboard.h>
#include <utils.h>

/* KBDUS means US Keyboard Layout. This is a scancode table
*  used to layout a standard US keyboard. I have left some
*  comments in to give you an idea of what key is what, even
*  though I set it's array index to 0. You can change that to
*  whatever you want using a macro, if you wish! */
const uint8_t kbdus[128] = {
    // 0x00
    0, 27, '1', '2', '3', '4', '5', '6',
    // 0x08
    '7', '8', '9', '0', '-', '=', '\b', '\t',
    // 0x10
    'q', 'w', 'e', 'r', 't', 'y', 'u', 'i',
    // 0x18, 0x1D - Control
    'o', 'p', '[', ']', '\n', 0, 'a', 's',
    // 0x20
    'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',
    // 0x28, 0x2A - Left Shift
    '\'', '`', 0, '\\', 'z', 'x', 'c', 'v',
    // 0x30, 0x36 - Right shift
    'b', 'n', 'm', ',', '.', '/', 0, '*',
    // 0x38 - Alt, 0x3A - Caps lock, 0x3B - F1
    0, ' ', 0, 0, 0, 0, 0, 0,
    // 0x40, 0x44 - F10, 0x45 - Num lock, 0x46 - Scroll lock, 0x47 - Home
    0, 0, 0, 0, 0, 0, 0, 0,
    // 0x48 - Up, 0x49 - Page up, 0x4B - Left, 0x4D - Right, 0x4F - End
    0, 0, '-', 0, 0, 0, '+', 0,
    // 0x50 - Down, 0x51 - Page Down, 0x52 - Insert, 0x53 - Delete, 0x57 - F11
    0, 0, 0, 0, 0, 0, 0, 0,
    // 0x58 - F12,
    0, 0,
    // All other keys are undefined
};

#define CTRL_KEY 0x1D
#define ALT_KEY 0x11
#define SHIFT_KEY 0x2A
#define BACKSPACE_KEY 0x66
#define CAPSLOCK_KEY 0x58
#define KEYBUFFER_LIMIT 1024
#define q_next(x) (((x) + 1) % KEYBUFFER_LIMIT)

bool pressed[128] = {0};
uint8_t keyboard_queue[KEYBUFFER_LIMIT];
uint64_t q_start = 0, q_end = 0;

// Agrega key al buffer de teclado si hay suficiente espacio
void keybuffer_produce(uint8_t key) {
    keyboard_queue[q_end] = key;
    q_end = q_next(q_end);
    // Si la cola esta llena perdemos caracteres viejos
    if (q_end == q_start)
        q_start = q_next(q_start);
}

// Saca del buffer de teclado
int64_t keybuffer_consume() {
    uint64_t rflags = cli_safe();
    int64_t res = -1;
    if (q_start != q_end) {
        res = (int64_t)keyboard_queue[q_start];
        q_start = q_next(q_start);
    }
    sti_safe(rflags);
    return res;
}

void handle_key_event(uint8_t scancode) {
    bool was_released = scancode & 0x80;
    uint8_t code = scancode & 0x7F;
    pressed[code] = !was_released;
    if (was_released)
        return;
    uint8_t key = kbdus[code];
    if (key)
        keybuffer_produce(key);
}
