#include <defines.h>
#include <keyboard.h>
#include <ports.h>
#include <screen.h>
#include <timer.h>
#include <utils.h>

// Array para acceder al video por caracter
#define VIDEO_ARRAY ((uint16_t(*)[VIDEO_WIDTH])VIDEO_ADDRESS)

static uint16_t format = SCREEN_FORMAT_DEFAULT;
static uint8_t cursor_r = 0, cursor_c = 0;

static void screen_set_blink_indicator(char row, char col) {
    uint16_t tmp = ((uint32_t)row) * VIDEO_WIDTH + col;
    outb(0x3D4, 14);
    outb(0x3D5, tmp >> 8);
    outb(0x3D4, 15);
    outb(0x3D5, tmp);
}

static void screen_clear_line(uint8_t row) {
    for (uint64_t i = 0; i < VIDEO_WIDTH; i++)
        VIDEO_ARRAY[row][i] = format | 0x20;
}

char screen_getrow() {
    return cursor_r;
}

char screen_getcol() {
    return cursor_c;
}

void screen_clear() {
    for (uint64_t i = 0; i < VIDEO_HEIGHT; i++)
        screen_clear_line(i);
    screen_setcursor(0, 0);
}

void screen_setcursor(uint8_t row, uint8_t col) {
    cursor_r = row;
    cursor_c = col;
    screen_set_blink_indicator(row, col);
}

void screen_setmode(uint8_t font, uint8_t back) {
    format = SCREEN_FORMAT(font, back);
}

void screen_move_back() {
    if (cursor_c > 0)
        screen_setcursor(cursor_r, cursor_c - 1);
    VIDEO_ARRAY[cursor_r][cursor_c] = 0x20 | format;
}

void screen_shift_up() {
    memmove(VIDEO_ARRAY, VIDEO_ARRAY + 1, VIDEO_SIZE);
    screen_clear_line(VIDEO_HEIGHT - 1);
    screen_setcursor(cursor_r - 1, cursor_c);
}

void screen_putchar(char c) {
    switch (c) {
        case '\n':
            screen_setcursor(cursor_r + 1, 0);
            break;
        case '\r':
            screen_clear_line(cursor_r);
            screen_setcursor(cursor_r, 0);
            break;
        case '\t':
            screen_setcursor(cursor_r, cursor_c + TAB_WIDTH);
            break;
        default:  // Caracter Imprimible
            VIDEO_ARRAY[cursor_r][cursor_c] = format | c;
            screen_setcursor(cursor_r, cursor_c + 1);
            break;
    }
    if (cursor_c >= VIDEO_WIDTH)
        screen_setcursor(cursor_r + 1, cursor_c - VIDEO_WIDTH);
    if (cursor_r >= VIDEO_HEIGHT)
        screen_shift_up();
}

uint64_t screen_read(void* buf, uint64_t count) {
    char* data = buf;
    int64_t c;
    uint64_t read = 0;
    while (read < count) {
        do {
            c = keybuffer_consume();
            sleep(1);
        } while (c == -1);
        if (c == '\n') {
            screen_putchar(c);
            break;
        } else if (c == '\b') {
            // TODO: Esto es super bugoso cuando hay multilinea
            if (read > 0) {
                read--;
                screen_move_back();
            }
        } else if (c == '\t') {
            // TAB es malo, se podria hacer que ponga espacios, no recomendable
        } else {
            data[read++] = c;
            screen_putchar(c);
        }
    }
    return read;
}

uint64_t screen_write(const void* buf, uint64_t count) {
    const char* start = buf;
    const char* end = start + count;
    while (start < end)
        screen_putchar(*start++);
    return count;
}

uint64_t screen_write_err(const void* buf, uint64_t count) {
    uint16_t temp_format = format;
    format = SCREEN_FORMAT_ERROR;
    uint64_t res = screen_write(buf, count);
    format = temp_format;
    return res;
}
