#include <command.h>
#include <io.h>
#include <loader.h>
#include <print.h>
#include <shell.h>

int64_t change_directory(uint64_t argc, char* argv[]) {
    if (argc == 0)
        return 0;
    if (argc == 1)
        return set_cwd("/");
    // Si es un path desde el root lo cargamos como viene
    if (argv[1][0] == '/')
        return set_cwd(argv[1]);
    return set_relative_cwd(argv[1]);
}

int64_t list_directory(uint64_t argc, char* argv[]) {
    const char* path;
    if (argc == 0) {
        return 0;
    } else if (argc == 1) {
        path = ".";
    } else if (argc == 2) {
        path = argv[1];
    } else {
        print("Se esperaba a lo sumo un argumento.\n");
        return -1;
    }
    int64_t dir = opendir(path);
    if (dir < 0) {
        print("No se pudo abrir el directorio.\n");
        return -1;
    }
    dir_entry* archivo;
    while ((archivo = readdir(dir)))
        printf("%s\n", archivo->name);
    closedir(dir);
    return 0;
}

int64_t echo_string(uint64_t argc, char* argv[]) {
    for (uint64_t i = 1; i < argc; i++)
        printf("%s ", argv[i]);
    return 0;
}

int64_t load_binary(uint64_t argc, char* argv[]) {
    if (argc < 2) {
        print("Se esperaba un argumento al menos.\n");
        return -1;
    }

    print("Abriendo imagen binaria en el filesystem.\n");
    int64_t file = open(argv[1]);
    if (file < 0) {
        print("Fallo en la apertura del archivo.\n");
        return -1;
    }
    print("Cargando...\n");
    // El loader siempre devuelve un puntero válido de application data.
    application_data *termination =
        load_exec_elf64(file, argc - 1, argv + 1);
    close(file);
    switch (termination->loader_error) {
        case 0:
            break;
        case INVALID_ELF:
            print("El archivo no es una imagen ELF valida.\n");
            return 0;
        case ERRREADING_ELF:
            print("Ocurrio un problema al leer el archivo ELF.\n");
            return 0;
        case BAD_LOADER_ACCESS:
            print("El loader intento acceder a una seccion inexistente.\n");
            return 0;
        default:
            print("Error desconocido.\n");
            return 0;
    }
    printf("Codigo de terminacion: %i\n", termination->exit_code);
    printf("Nucleo que termino la aplicacion: %i\n", termination->exit_core_id);
    return 0;
}
