#include <core_id.h>
#include <i386.h>
#include <ports.h>
#include <timer.h>

static volatile uint64_t sleep_counter[MAX_PROCESSORS] = {0};

void timer_init() {
    // Leemos el estado actual del registro B del RTC
    uint8_t rct = cmos_readb(0x0B);
    // Seteamos el bit de periódico de dicho registro
    cmos_writeb(0x0B, rct | (1 << 6));
}

void timer_tick() {
    // TODO: Cada procesador deberia restar su timer
    // Simulamos "lock" con cli y sti
    uint64_t flags = cli_safe();
    for (uint64_t i = 0; i < MAX_PROCESSORS; i++)
        if (sleep_counter[i])
            sleep_counter[i]--;
    sti_safe(flags);
}

void sleep(uint64_t ticks) {
    // Simulamos "lock" con cli y sti
    uint64_t flags = cli_safe();
    uint64_t id = core_id();
    sleep_counter[id] = ticks;
    sti_safe(flags);
    // TODO: cambiar por hlt jmp cmp
    // Sleep until clock sets the counter to 0
    while (sleep_counter[id]);
}
