#include <exception.h>
#include <i386.h>
#include <print.h>
#include <screen.h>
#include <utils.h>

extern symbol_entry SYMBOL_TABLE[];
extern uint64_t SYMBOL_COUNT;

symbol_entry* get_symbol_entry(uintptr_t fun_addr) {
    uint64_t i;
    intptr_t min_offset = 0;
    int64_t min_index = -1;
    for (i = 0; i < SYMBOL_COUNT; i++) {
        intptr_t offset = fun_addr - SYMBOL_TABLE[i].addr;
        if (offset >= 0 && (offset < min_offset || min_index == -1)) {
            min_offset = offset;
            min_index = i;
        }
    }
    if (min_index >= 0)
        return SYMBOL_TABLE + min_index;
    return NULL;
}

void calltrace(uint64_t count) {
    uintptr_t* base = (uintptr_t*)getRBP();
    print("\n[CALL TRACE] (Puede haber funciones mal traceadas)\n");
    print("Function                 Address\n");
    while (base && count) {
        uintptr_t fun_addr = base[1];
        symbol_entry* symbol = get_symbol_entry(fun_addr);
        if (!symbol)
            printf("Unknown Function         0x%p\n", fun_addr);
        else {
            uint64_t len = strlen(symbol->name);
            printf("%s + %p", symbol->name, fun_addr - symbol->addr);
            for (uint64_t i = 0; i < 20 - len; i++)
                print(" ");
            printf("0x%p\n", fun_addr);
        }
        base = (uintptr_t*)*base;
        count--;
    }
}

void kernel_panic(const char* function, const uint64_t line, const char* file,
                  const char* message, ...) {
    va_list l;
    screen_setmode(COLOR_WHITE, COLOR_BLUE);
    screen_clear();
    printf("[KERNEL PANIC]: ");
    va_start(l, message);
    vprintf(message, l);
    va_end(l);

    printf("\n\nFuncion que produjo el error:\t%s\n", function);
    printf("(Archivo:Linea)\t%s:%d\n", file, line);

    calltrace(40);
    __asm __volatile("xchg %%bx, %%bx" : :);
    hlt();
}
