#include <utils.h>

void* memcpy(void* dst, const void* src, uint64_t count) {
    uint64_t* dst64 = (uint64_t*)dst;
    uint64_t* src64 = (uint64_t*)src;
    while (count >= sizeof(uint64_t)) {
        *dst64++ = *src64++;
        count -= sizeof(uint64_t);
    }
    // Ahora que nos sobraron bytes los copiamos tambien
    char* dst8 = (char*)dst64;
    char* src8 = (char*)src64;
    while (count--)
        *dst8++ = *src8++;
    return dst;
}

void* memmove(void* dst, const void* src, uint64_t count) {
    if (dst <= src) {
        memcpy(dst, src, count);
    } else {
        uint64_t* dst64 = (uint64_t*)((uintptr_t)dst + count);
        uint64_t* src64 = (uint64_t*)((uintptr_t)src + count);
        while (count >= sizeof(uint64_t)) {
            *(--dst64) = *(--src64);
            count -= sizeof(uint64_t);
        }
        // Ahora que nos sobraron bytes los copiamos tambien
        char* dst8 = (char*)dst64;
        char* src8 = (char*)src64;
        while (count--) {
            *(--dst8) = *(--src8);
        }
    }
    return dst;
}

void* memset(void* dst, uint8_t value, uint64_t length) {
    uint8_t* ptr = (uint8_t*)dst;
    uint8_t* end = dst + length;
    while (ptr != end)
        *ptr++ = value;
    return dst;
}

uint64_t memcmp(const void* m1, const void* m2, uint64_t bytes) {
    const char* str_m1 = m1;
    const char* str_m2 = m2;
    for (uint64_t i = 0; i < bytes; i++) {
        if (str_m1[i] < str_m2[i])
            return -1;
        if (str_m1[i] > str_m2[i])
            return 1;
    }
    return 0;
}

uint64_t strlen(const char* str) {
    uint64_t count;
    for (count = 0; str[count]; count++);
    return count;
}

int64_t atoll(const char* string) {
    bool sign = false;
    if (*string == '-') {
        sign = true;
        string++;
    }
    uint64_t res = 0;
    while (string) {
        if ((*string < '0') || (*string > '9'))
            return 0;
        res *= 10;
        res += *string - '0';
        string++;
    }
    return (sign ? -res : res);
}

void itoa(int64_t number, char* output) {
    uint64_t i = 0;
    bool sign = false;
    if (number < 0) {
        number = -number;
        sign = true;
    }
    do {
        output[i++] = (number % 10) + '0';
        number /= 10;
    } while (number > 0);
    if (sign)
        output[i++] = '-';
    output[i] = '\0';
    strnrev(output, i);
}

void itoa_log(uint64_t number, char* output, uint64_t base_log) {
    uint64_t i = 0;
    do {
        char c = number & ((1 << base_log) - 1);
        if (c > 9)
            c += 'A' - 10;
        else
            c += '0';
        output[i++] = c;
        number >>= base_log;
    } while (number);
    output[i] = '\0';
    strnrev(output, i);
}

void strrev(char* s) {
    strnrev(s, strlen(s));
}

void strnrev(char* s, uint64_t n) {
    for (char *end = s + n - 1; s < end; s++, end--) {
        char c = *s;
        *s = *end;
        *end = c;
    }
}

char* strcpy(char* dst, const char* src) {
    for (uint64_t i = 0; (dst[i] = src[i]); i++);
    return dst;
}

char* strncpy(char* dst, const char* src, uint64_t len) {
    uint64_t i;
    for (i = 0; i < len && (dst[i] = src[i]); i++);
    while (i < len) {
        dst[i] = 0;
        i++;
    }
    return dst;
}

char* strcat(char* dst, const char* src) {
    strcpy(dst + strlen(dst), src);
    return dst;
}

char* strncat(char* dst, const char* src, uint64_t len) {
    char* start = dst + strlen(dst);
    uint64_t i;
    for (i = 0; i < len && src[i]; i++)
        start[i] = src[i];
    start[i] = 0;
    return dst;
}

int64_t strncmp(const char* str1, const char* str2, uint64_t n) {
    for (uint64_t i = 0; i < n; i++) {
        if (str1[i] < str2[i])
            return -1;
        if (str1[i] > str2[i])
            return 1;
        if (!str1[i])
            break;
    }
    return 0;
}

int64_t strcmp(const char* str1, const char* str2) {
    do {
        if (*str1 < *str2)
            return -1;
        if (*str1 > *str2)
            return 1;
    } while (str1++, *str2++);
    return 0;
}

// Hace el checksum en bytes.
char byte_checksum(const void* buf, unsigned int len) {
    const char* bytes = (const char*)buf;
    const char* end = bytes + len;
    char sum = 0;
    while (bytes != end)
        sum += *bytes++;
    return sum;
}
