#ifndef __SYSCALL_H__
#define __SYSCALL_H__

#include <stdint.h>

struct syscall_table {
    void (*sys_exit) (int64_t error_code);
    uint64_t (*sys_core_id) (void);
    uint64_t (*sys_core_count) (void);
    void    (*sys_sleep) (uint64_t ticks);
    int64_t (*sys_create) (char* path, uint8_t type);
    int64_t (*sys_open) (char* path);
    int64_t (*sys_close) (int64_t fd);
    uint64_t (*sys_read) (int64_t fd, void *buf, uint64_t count);
    uint64_t (*sys_write) (int64_t fd, void *buf, uint64_t count);
    int64_t (*sys_seek) (int64_t fd, int64_t offset, int64_t origin);
    int64_t (*sys_tell) (int64_t fd);
    int64_t (*sys_type) (char *path);
    int64_t (*sys_opendir) (char *path);
    int64_t (*sys_closedir) (int64_t dd);
    int64_t (*sys_rewinddir) (int64_t dd);
};

extern struct syscall_table SYSCALL_TABLE;

#define exit(error_code)            SYSCALL_TABLE.sys_exit(error_code)
#define core_id()                   SYSCALL_TABLE.sys_core_id()
#define core_count()                SYSCALL_TABLE.sys_core_count()
#define sleep(ticks)                SYSCALL_TABLE.sys_sleep(ticks);
#define create(path, type)          SYSCALL_TABLE.sys_create(path, type)
#define open(path)                  SYSCALL_TABLE.sys_open(path)
#define close(fd)                   SYSCALL_TABLE.sys_close(fd)
#define read(fd, buf, count)        SYSCALL_TABLE.sys_read(fd, buf, count)
#define write(fd, buf, count)       SYSCALL_TABLE.sys_write(fd, buf, count)
#define seek(fd, offset, origin)    SYSCALL_TABLE.sys_seek(fd, offset, origin)
#define tell(fd)                    SYSCALL_TABLE.sys_tell(fd)
#define type(path)                  SYSCALL_TABLE.sys_type(path)
#define opendir(path)               SYSCALL_TABLE.sys_opendir(path)
#define closedir(dd)                SYSCALL_TABLE.sys_closedir(dd)
#define rewinddir(dd)               SYSCALL_TABLE.sys_rewinddir(dd)

#endif
